import { cojsonInternals, LocalNode, } from "cojson";
import { Account } from "../internal.js";
export const fixedCredentialsAuth = (credentials) => {
    return {
        start: async () => ({
            type: "existing",
            credentials,
            onSuccess: () => { },
            onError: () => { },
        }),
    };
};
export async function randomSessionProvider(accountID) {
    return {
        sessionID: cojsonInternals.newRandomSessionID(accountID),
        sessionDone: () => { },
    };
}
export async function createJazzContext({ AccountSchema = Account, auth, sessionProvider, peersToLoadFrom, crypto, }) {
    // eslint-disable-next-line no-constant-condition
    while (true) {
        const authResult = await auth.start(crypto);
        if (authResult.type === "existing") {
            try {
                const { sessionID, sessionDone } = await sessionProvider(authResult.credentials.accountID);
                try {
                    const node = await LocalNode.withLoadedAccount({
                        accountID: authResult.credentials
                            .accountID,
                        accountSecret: authResult.credentials.secret,
                        sessionID: sessionID,
                        peersToLoadFrom: peersToLoadFrom,
                        crypto: crypto,
                        migration: async (rawAccount, _node, creationProps) => {
                            const account = new AccountSchema({
                                fromRaw: rawAccount,
                            });
                            await account.migrate?.(creationProps);
                        },
                    });
                    const account = AccountSchema.fromNode(node);
                    authResult.onSuccess();
                    return {
                        account,
                        done: () => {
                            node.gracefulShutdown();
                            sessionDone();
                        },
                    };
                }
                catch (e) {
                    authResult.onError(new Error("Error loading account", { cause: e }));
                    sessionDone();
                }
            }
            catch (e) {
                authResult.onError(new Error("Error acquiring sessionID", { cause: e }));
            }
        }
        else if (authResult.type === "new") {
            try {
                // TODO: figure out a way to not "waste" the first SessionID
                const { node } = await LocalNode.withNewlyCreatedAccount({
                    creationProps: authResult.creationProps,
                    peersToLoadFrom: peersToLoadFrom,
                    crypto: crypto,
                    initialAgentSecret: authResult.initialSecret,
                    migration: async (rawAccount, _node, creationProps) => {
                        const account = new AccountSchema({
                            fromRaw: rawAccount,
                        });
                        await account.migrate?.(creationProps);
                    },
                });
                const account = AccountSchema.fromNode(node);
                await authResult.saveCredentials({
                    accountID: node.account.id,
                    secret: node.account.agentSecret,
                });
                authResult.onSuccess();
                return {
                    account,
                    done: () => {
                        node.gracefulShutdown();
                    },
                };
            }
            catch (e) {
                authResult.onError(new Error("Error creating account", { cause: e }));
            }
        }
    }
}
//# sourceMappingURL=createContext.js.map